==> rails_6.0.3.7+dfsg.orig.tar.xz: rails-6.0.3.7/activerecord/test/cases/adapters/postgresql/optimizer_hints_test.rb <==
# frozen_string_literal: true

require "cases/helper"
require "models/post"

if supports_optimizer_hints?
  class PostgresqlOptimzerHintsTest < ActiveRecord::PostgreSQLTestCase
    fixtures :posts

    def setup
      enable_extension!("pg_hint_plan", ActiveRecord::Base.connection)
    end

    def test_optimizer_hints
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("SeqScan(posts)")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_includes posts.explain, "Seq Scan on posts"
      end
    end

    def test_optimizer_hints_with_count_subquery
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("SeqScan(posts)")
        posts = posts.select(:id).where(author_id: [0, 1]).limit(5)
        assert_equal 5, posts.count
      end
    end

    def test_optimizer_hints_is_sanitized
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("/*+ SeqScan(posts) */")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_includes posts.explain, "Seq Scan on posts"
      end

      assert_sql(%r{\ASELECT /\*\+  "posts"\.\*,  \*/}) do
        posts = Post.optimizer_hints("**// \"posts\".*, //**")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_equal({ "id" => 1 }, posts.first.as_json)
      end
    end

    def test_optimizer_hints_with_unscope
      assert_sql(%r{\ASELECT "posts"\."id"}) do
        posts = Post.optimizer_hints("/*+ SeqScan(posts) */")
        posts = posts.select(:id).where(author_id: [0, 1])
        posts.unscope(:optimizer_hints).load
      end
    end
  end
end

==> rails_6.1.7.10+dfsg.orig.tar.xz: rails-6.1.7.10/activerecord/test/cases/adapters/postgresql/optimizer_hints_test.rb <==
# frozen_string_literal: true

require "cases/helper"
require "models/post"

if supports_optimizer_hints?
  class PostgresqlOptimzerHintsTest < ActiveRecord::PostgreSQLTestCase
    fixtures :posts

    def setup
      enable_extension!("pg_hint_plan", ActiveRecord::Base.connection)
    end

    def test_optimizer_hints
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("SeqScan(posts)")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_includes posts.explain, "Seq Scan on posts"
      end
    end

    def test_optimizer_hints_with_count_subquery
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("SeqScan(posts)")
        posts = posts.select(:id).where(author_id: [0, 1]).limit(5)
        assert_equal 5, posts.count
      end
    end

    def test_optimizer_hints_is_sanitized
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        posts = Post.optimizer_hints("/*+ SeqScan(posts) */")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_includes posts.explain, "Seq Scan on posts"
      end

      assert_sql(%r{\ASELECT /\*\+  "posts"\.\*,  \*/}) do
        posts = Post.optimizer_hints("**// \"posts\".*, //**")
        posts = posts.select(:id).where(author_id: [0, 1])
        assert_equal({ "id" => 1 }, posts.first.as_json)
      end
    end

    def test_optimizer_hints_with_unscope
      assert_sql(%r{\ASELECT "posts"\."id"}) do
        posts = Post.optimizer_hints("/*+ SeqScan(posts) */")
        posts = posts.select(:id).where(author_id: [0, 1])
        posts.unscope(:optimizer_hints).load
      end
    end

    def test_optimizer_hints_with_or
      assert_sql(%r{\ASELECT /\*\+ SeqScan\(posts\) \*/}) do
        Post.optimizer_hints("SeqScan(posts)").or(Post.all).load
      end

      queries = capture_sql do
        Post.optimizer_hints("SeqScan(posts)").or(Post.optimizer_hints("IndexScan(posts)")).load
      end
      assert_equal 1, queries.length
      assert_includes queries.first, "/*+ SeqScan(posts) */"
      assert_not_includes queries.first, "/*+ IndexScan(posts) */"

      queries = capture_sql do
        Post.all.or(Post.optimizer_hints("IndexScan(posts)")).load
      end
      assert_equal 1, queries.length
      assert_not_includes queries.first, "/*+ IndexScan(posts) */"
    end
  end
end
